unit acmwave;

interface

uses sysutils, messages, msacm, mmsystem, dialogs, windows, forms;

type
  TOnData = procedure(data:pointer; size:longint);
  TACMWaveOut = class
     private
       closed: boolean;
       HWaveOut1:PHWaveOut;
       handle: HWND;
       procedure WaveOutCallback(var msg:TMessage); message MM_WOM_DONE;
     public
       function Open(format:PWaveFormatEx): integer;
       procedure PlayBack(data:pointer;size:longint);
       procedure Close;
       constructor Create;
     end;

  TACMWaveIn = class
    private
      closed: boolean;
      sizebuf: integer;
      HWaveIn1: PHWaveIn;
      handle: HWND;
    procedure WaveInCallback (var msg:TMessage); message MM_WIM_DATA;
      { Private declarations }
    public
      FOnData: TOnData;
      function Open(format:PWaveFormatEx): integer;
      procedure Close;
    constructor Create;
    end;

implementation

constructor TACMWaveOut.Create;
begin
 closed:=true;
 HWaveOut1:=nil;
end;

function TACMWaveOut.Open(format:PWaveFormatEx): integer;
var
 waveformat: PWaveFormatEx;
 maxsizeformat,i: integer;
begin
 i:=0;
 if (format<>nil) and (HWaveOut1=nil) then begin
     handle:=Application.Handle;
     acmMetrics(0, ACM_METRIC_MAX_SIZE_FORMAT,MaxSizeFormat);
     getmem(WaveFormat, MaxSizeFormat);
     move(format^,waveformat^,maxsizeformat);
     HWaveOut1:=new(PHWaveOut);
     //create playing handle with waveformatex structure
     i:=WaveOutOpen(HWaveOut1,0,waveformat,handle,0,CALLBACK_WINDOW or WAVE_MAPPED);
     if i<>0 then begin
       result:=i;
       exit;
       end;
     closed:=false;
     end;
 result:=i;
end;


procedure TACMWaveOut.PlayBack(data:pointer;size:longint);
var
 Header: PWaveHdr;
 memblock: pointer;
 i: integer;
begin
 if HWaveOut1<>nil then begin
    header:=new(PWaveHdr);
    //memblock:=new(pointer);
    getmem(memblock,size);
    move(data^,memBlock^,size);
    header.lpdata:=memBlock;
    header.dwbufferlength:=size;
    header.dwbytesrecorded:=size;
    header.dwUser:=0;
    header.dwflags:=0;
    header.dwloops:=0;
    i:=WaveOutPrepareHeader(HWaveOut1^,header,sizeof(TWaveHdr));
    if i<> 0 then showmessage('WaveOutPrepareHeader error');
    i:=WaveOutWrite(HWaveOut1^,header,sizeof(TWaveHdr));
    if i<> 0 then showmessage('WaveOutWrite error');
    end;
end;

procedure TACMWaveOut.WaveOutCallback(var msg:TMessage);
var
 header: PWaveHdr;
 i: integer;
begin
 header:=PWaveHdr(msg.LParam);
 if closed=false then
 begin
 i:=WaveOutUnPrepareHeader(HWaveOut1^,header,sizeof(TWaveHdr));
 if i<> 0 then showmessage('WaveOutPrepareHeader error');
 end;
 dispose(Header^.lpData);
 dispose(Header);
end;

procedure TACMWaveOut.Close;
begin
 if HWaveOut1<>nil then
 begin
 closed:=TRUE;
 WaveOutReset(HWaveOut1^);
 WaveOutClose(HWaveOut1^);
 HWaveOut1:=nil;
 end;
end;


procedure TACMWaveIn.WaveInCallback(var msg:TMessage);
// called on buffer full
var
   Header:PWaveHdr;
   i,bytesrecorded:integer;
   data:PChar;
begin
     {block has been recorded}
     Header:=PWaveHdr(msg.lparam);
     if closed=false then
     begin
     i:=waveInUnPrepareHeader(HWaveIn1^,Header,sizeof(TWavehdr));
     if i<>0 then showmessage('In Un Prepare error');
     bytesrecorded:=header.dwbytesrecorded;
     getmem(data,bytesrecorded); //allocate memory
     move(header.lpdata^,data^,bytesrecorded); //copy data
     if assigned(FOnData) then
     begin
      FOnData(data,bytesrecorded);
     end;

          Freemem(data); //free memory
          {reuse a old memory block}

          header.dwbufferlength:=sizebuf;
          header.dwbytesrecorded:=0;
          header.dwUser:=0;
          header.dwflags:=0;
          header.dwloops:=0;

         {prepare the old block}
          i:=waveInPrepareHeader(HWaveIn1^,Header,sizeof(TWavehdr));

          if i<>0 then showmessage('In Prepare error');

          {add it to the buffer}
          i:=waveInAddBuffer(HWaveIn1^,Header,sizeof(TWaveHdr));
          if i<>0 then showmessage('Add buffer error');
         end
         else
         begin //free buffers if closed
          dispose(header.lpdata);
          dispose(header);
         end;
end;

constructor TACMWaveIn.Create;
begin
 hwavein1:=nil;
 closed:=true;
end;


function TACMWaveIn.Open(format:PWaveFormatEx): integer;
var
   WaveFormat: PWaveFormatEx;
   Header: PWaveHdr;
   memBlock: PChar;
   i,j,maxsizeformat: integer;
begin
  i:=0;
  if (hwavein1=nil) and (format<>nil) then begin
     handle:=Application.Handle;
     acmMetrics(0, ACM_METRIC_MAX_SIZE_FORMAT,MaxSizeFormat);
     getmem(WaveFormat, MaxSizeFormat);
     move(format^,waveformat^,maxsizeformat);
     sizebuf:=format.nAvgBytesPerSec;
     HWaveIn1:=new(PHWaveIn);
     // create record handle with waveformatex structure
     i:=WaveInOpen(HWaveIn1,0,waveformat,handle,0,CALLBACK_WINDOW or WAVE_MAPPED);
     if i<>0 then begin
       // showmessage('Problem creating record handle' + inttostr(i));
       result:=i;
       exit;
     end;
     closed:=false;
     {need to add some buffers to the recording queue}
     {in case the messages that blocks have been recorded}
     {are delayed}
     for j:= 1 to 3 do
     begin
          {make a new block}
          Header:=new(PWaveHdr);
//          memBlock:=new(PChar);
          getmem(memblock,sizebuf); //allocate memory
          header.lpdata:=memBlock;
          header.dwbufferlength:=sizebuf;
          header.dwbytesrecorded:=0;
          header.dwUser:=0;
          header.dwflags:=0;
          header.dwloops:=0;
          {prepare the new block}
          i:=waveInPrepareHeader(HWaveIn1^,Header,sizeof(TWavehdr));
          if i<>0 then showmessage('In Prepare error');
          {add it to the buffer}
          i:=waveInAddBuffer(HWaveIn1^,Header,sizeof(TWaveHdr));
          if i<>0 then showmessage('Add buffer error');
     end; {of loop}

     {finally start recording}
     i:=waveInStart(HwaveIn1^);
     if i<>0 then showmessage('Start error');
  end;
  result:=i;
end;

procedure TACMWaveIn.Close;
begin
 if HWaveIn1<>nil then begin
   closed:=true;
   WaveInReset(HWaveIn1^);
   WaveInClose(HWaveIn1^);
   dispose(HWaveIn1);
   HWaveIn1:=nil;
   end;
end;

end.
